#!/usr/bin/python
# -*- coding: utf-8 -*-

# Author: Anton Trapeznikov <anton.trapeznikov@gmail.com>
# License: MIT License

import sys
import argparse


def to_float(val, default=None):
    value = default

    if val:
        if isinstance(val, (str, unicode)):
            val = val.replace(',', '.')

        try:
            value = float(val)
        except Exception:
            value = default

    return value


def to_int(val, default=None):
    value = default

    if val:
        if isinstance(val, (float)):
            val = val.replace(',', '.')

        try:
            value = float(val)
        except Exception:
            value = default

    return value


def get_from_dict(source, key, default=None):
    return source[key] if key in source else default


class Usage(Exception):
    def __init__(self, msg):
        self.msg = msg


class SpiritProcessor(object):
    def __init__(self, volume, boiler_alc=None, first_drops_alc=None):
        self._liquid_and_steam = {
            0.25: 2.53,
            0.51: 5.08,
            0.76: 7.56,
            1.02: 10.1,
            1.27: 12.52,
            1.52: 15.0,
            1.77: 17.18,
            1.99: 19.4,
            2.27: 21.36,
            2.52: 22.87,
            3.01: 25.88,
            3.5: 28.7,
            3.99: 31.12,
            4.48: 33.66,
            4.97: 35.98,
            5.44: 38.15,
            5.92: 40.19,
            6.39: 42.06,
            6.86: 43.77,
            7.34: 45.25,
            8.5: 48.99,
            9.64: 51.15,
            10.77: 54.33,
            11.88: 55.12,
            13.07: 56.86,
            14.04: 58.49,
            14.9: 59.62,
            16.61: 60.80,
            17.2: 61.9,
            18.2: 62.93,
            19.2: 63.91,
            20.2: 64.84,
            21.62: 65.39,
            22.15: 66.50,
            23.1: 67.22,
            24.0:  67.91,
            24.87: 68.58,
            25.88: 69.19,
            26.78: 69.74,
            27.67: 70.27,
            28.55: 70.6,
            29.42: 71.75,
            30.28: 71.40,
            31.12: 71.73,
            31.96: 71.91,
            32.78: 72.43,
            33.68: 72.62,
            34.40: 72.85,
            35.2: 73.05,
            35.98: 73.28,
            36.76: 73.5,
            37.52: 73.71,
            38.28: 73.9,
            39.11: 74.1,
            40.5: 74.48,
            41.93: 74.82,
            43.53: 75.16,
            44.71: 75.46,
            46.05: 75.8,
            47.41: 76.09,
            49.02: 76.42,
            49.89: 76.75,
            51.12: 77.08,
            52.39: 77.40,
            53.49: 77.69,
            54.72: 77.96,
            55.74: 78.21,
            56.86: 78.49,
            57.96: 78.77,
            59.02: 79.01,
            60.06: 79.31,
            61.07: 79.54,
            62.08: 79.79,
            63.06: 80.01,
            64.02: 80.24,
            64.8: 80.47,
            65.88: 80.70,
            66.80: 80.96,
            67.70: 81.19,
            68.56: 81.45,
            69.42: 81.67,
            70.27: 81.94,
            71.1: 82.19,
            71.91: 82.45,
            72.71: 82.73,
            73.5: 83.01,
            74.27: 83.29,
            75.03: 83.61,
            75.79: 83.91,
            76.50: 84.21,
            77.24: 84.52,
            77.95: 84.85,
            78.65: 85.18,
            79.34: 85.53,
            80.02: 85.82,
            80.69: 86.17,
            81.0: 86.49,
            82.0: 86.8,
            82.63: 87.11,
            83.25: 87.42,
            83.87: 87.73,
            84.47: 88.03,
            85.07: 88.33,
            85.66: 88.71,
            86.24: 88.96,
            86.82: 89.28,
            87.38: 89.59,
            87.94: 89.91,
            88.48: 90.22,
            89.02: 90.53,
            89.55: 90.84,
            90.08: 91.17,
            90.6: 91.5,
            91.11: 91.87,
            91.61: 92.22,
            92.1: 92.59,
            92.6: 92.99,
            93.1: 93.43,
            93.5: 93.82,
            94.02: 94.23,
            94.49: 94.62,
            94.95: 94.96,
            95.39: 95.41,
            95.84: 95.79,
            96.28: 96.17,
            96.67: 96.53,
        }

        self._volume = to_float(volume)
        self._boiler_alc = to_float(boiler_alc)
        self._first_drops_alc = to_float(first_drops_alc)
        self._alcohol_volume = None

        if not self._volume:
            raise ValueError(u'Нет корректных данных об объеме жидкости в кубе')

        if boiler_alc is None and first_drops_alc is None:
            raise ValueError(u'Нет корректных данных о спиртуозности в кубе или спиртуозности первых капель')
        elif boiler_alc and not SpiritProcessor.validate_alc(boiler_alc):
            raise ValueError(u'Спиртуозность в кубе указана неправильно')
        elif first_drops_alc and not SpiritProcessor.validate_alc(first_drops_alc):
            raise ValueError(u'Спиртуозность первых капель указана неправильно')

        self._boiler_alc = self._boiler_alc or self._get_liquid_by_steam(first_drops_alc)
        self._alcohol_volume = 1000 * self._volume * self._boiler_alc / 100

    def _get_liquid_by_steam(self, steam_value): 
        distances = ((in_liquid, abs(in_steam - steam_value)) for in_liquid, in_steam in self._liquid_and_steam.items())
        return min(distances, key=lambda x:x[1])[0]

    def get_chunks(self, size): 
        if SpiritProcessor.validate_chunk_size(size):
            return [self._alcohol_volume * to_float(c) / 100 for c in size.split('-')]

        raise ValueError(u'Удельные доли спирта указаны неправильно')

    @staticmethod
    def validate_alc(alc_value):
        val = to_float(alc_value)
        return val and 0 < val < 100

    @staticmethod
    def validate_chunk_size(size):
        is_valid = False

        if size and isinstance(size, (str, unicode)):
            chunks = size.split('-')
            float_chunks = [to_float(c) for c in chunks if to_float(c) is not None]
            is_valid = len(float_chunks) == len(chunks) and len(chunks) > 0
            is_valid = is_valid and sum(float_chunks) == 100

        return is_valid

    @property
    def alcohol(self):
        return self._alcohol_volume

    @property
    def boiler_alc(self):
        return self._boiler_alc


def main(args=None):
    try:
        if args is None:
            parser = argparse.ArgumentParser()
            
            parser.add_argument('--boiler-volume', '-bv', required=True, type=str, default='', help=u'Объем жидкости в кубе. Литры.')
            parser.add_argument('--boiler-alc', '-ba', type=str, default='', help=u'Крепость навалки в кубе. Проценты.')
            parser.add_argument('--first-drops-alc', '-fda', type=str, default='', help=u'Спиртуозность первых капель. Проценты.')
            parser.add_argument('--chunk-size', '-cz', required=True, type=str, default='', help=u'Удельные доли АС. Доли отделяются дефисом. Сумма должна быть равна 100')

            options = parser.parse_args().__dict__
        else:
            options = args

        boiler_volume = to_float(get_from_dict(options, 'boiler_volume'))
        boiler_alc = to_float(get_from_dict(options, 'boiler_alc'))
        first_drops_alc = to_float(get_from_dict(options, 'first_drops_alc'))
        chunk_size = get_from_dict(options, 'chunk_size')

        if not boiler_volume:
            print u'Не указан параметр --boiler-volume (объем жидкости в кубе) или указан неправильно'
            return 3
        elif not chunk_size or SpiritProcessor.validate_chunk_size(chunk_size) == False:
            print u'Параметр --chunk-size (удельные доли АС) не указан или указан неправильно. Пример: 10-80-10'
            return 3
        elif boiler_alc == first_drops_alc == None:
            print u'Необходим один из параметров: --boiler-alc, --first-drops-alc'
            return 3
        elif boiler_alc and not SpiritProcessor.validate_alc(boiler_alc):
            print u'Параметр --boiler-alc (крепость навалки в кубе) указан неправильно. Пример: 40 или 38.5'
            return 3
        elif first_drops_alc and not SpiritProcessor.validate_alc(first_drops_alc):
            print u'Параметр --first-drops-alc (спиртуозность первых капель) указан неправильно. Пример: 40 или 38.5'
            return 3

        spirit = SpiritProcessor(volume=boiler_volume, boiler_alc=boiler_alc, first_drops_alc=first_drops_alc)

        print u'####################################'
        print u'Спиртуозность в кубе, проценты: %s' % spirit.boiler_alc
        print u'Абсолютного спирта в кубе, мл: %s' % int(round(spirit.alcohol))
        print u'Доли абсолютного спирта, мл: %s' % [int(round(c)) for c in spirit.get_chunks(chunk_size)] 
        print u'####################################'
    except Usage, err:
        print >>sys.stderr, err.msg
        print >>sys.stderr, u'Для справки испольуйте --help'
        return 2


if __name__ == "__main__":
    sys.exit(main())